%%%This script allows the reproduction of Wadkin LE, et al. Quantifying Invasive Pest Dynamics through Inference of a Two-Node Epidemic Network Model. Diversity. 2023; 15(4):496. https://doi.org/10.3390/d15040496

%%load the observed data set containing the removal prevalence data in Richmond and Bushy Park between 2013 and 2021
load RichmondBushy_13_21.mat %data from 2013 to 2021

%%time series plot of removal prevalence (Figure 1c)
time=2013:1:2021;
figure
plot(time,RichmondData13_21,'-o','Color',[0.8500, 0.3250, 0.0980], 'Linewidth',1.2,'MarkerFaceColor','auto','MarkerSize',5,'MarkerFaceColor',[0.8500, 0.3250, 0.0980])
hold on 
plot(time,BushyData13_21,'-d','Color',[0 0.4470 0.7410], 'Linewidth',1.2,'MarkerSize',6)
xlabel('Years')
ylabel('{\it R}({\itt})')
xlim([2013 2021])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%INFERENCE

%%specify data for Node 1 and Node 2 in the epidemic regime
Rtnode1=BushyData13_21; %Node 1 = Bushy 
Rtnode2=RichmondData13_21; %Node 2 = Richmond 

%%estimation at total N (numbers of trees) for each park (taken from previous work: doi https://doi.org/10.1002/ece3.8871)
Nnode1=5000;
Nnode2=40000;

%%S+I data for each node from N-Rt
SplusInode1=Nnode1-Rtnode1;
SplusInode2=Nnode2-Rtnode2;

%%observed data matrix
data=[SplusInode1; SplusInode2]';

%%estimation of I_0 (initial infected) for each node (taken from previous work: doi https://doi.org/10.1002/ece3.8871)
I0node1=240;
I0node2=1400;

%%logprior assuming normal priors
logprior=@(param,a,b) sum(log(normpdf(param,a,b)));
a=[0 0 0 0 0]; b=[1 1 1 1 1]; %log prior parameters - keep vague

%%initialise paramter chain for MCMC
theta=log([0.00001,0.8,0.3,2.5,1]); %log scale, initial parameter choices

%%timestep set up for inference (inter=inter observation time) and ODE timestep in the LNA
inter=1;
ODEdt=0.01;

%%initial conditions for time stepping forward the ODE (in LNA)
x0=[Nnode1-I0node1-Rtnode1(1), I0node1, Nnode2-I0node2-Rtnode2(1), I0node2]; %initial state
C=zeros(4,4); %initial variance

%%tuning parameter (found to be successful after an initial run and using 2.38^2/5*cov(theta))
load tuning.mat

%%Metropolis-Hastings set up
count=0; %tracking acceptance
iters=11000; %number of interations to perform
burnin=1000; %burn in

%%observation matrix to indicate which stats are observed
obsmatSIR=[1,1,0,0;0 0 1 1]';

%%current theta 
thetacurr=theta;
currloglike=-10000000; %set current loglike to be small initially to accept first iter 

%%run MCMC scheme
for i=2:iters
    thetaproposed=mvnrnd(thetacurr',tuning); %propose new theta
    param=exp(thetaproposed); %params for forward filter scheme
    
    %forward filter (using LNA to time step ODE) to get posteriors and proposed loglikelihood
    [xsamp,vhat,proposedloglike]=ffilter_2node(ODEdt,inter,param,x0,C,obsmatSIR,data);

    %reject any bad proposed loglikelihoods
    if isnan(proposedloglike) || proposedloglike==-1e40 
        alpha=-1e40;
    else
    proposedpost=proposedloglike+logprior(thetaproposed,a,b); %proposed posterior
    currpost=currloglike+logprior(thetacurr,a,b); %current posterior
    alpha=proposedpost-currpost;
    end
    
    u = rand(1);
    if log(u)<alpha %then update theta 
        thetacurr=thetaproposed; %update theta
        currloglike=proposedloglike; %update current loglikelihood
        xcan=xsamp; %update posteriors
        count = count+1; %track number of acceptances
    end

    xsampsave(:,:,i-1)=xcan; %saving within-sample posteriors
    theta(i,:)=thetacurr; %saving theta parameters
end

%%acceptance rate 
percentageacceptance=count/(iters)*100;

%%remove the burn in 
theta(1:burnin,:)=[];

%%parameter results
betaresults=exp(theta(:,1));
gammaresults=exp(theta(:,2));
a12results=exp(theta(:,3));
a21results=exp(theta(:,4));
sigeresults=exp(theta(:,5));

%%plotting the posterior parameter trace plots
figure
subplot(1,5,1)
plot(betaresults)
hold on
xlabel('iters')
ylabel('\beta')
subplot(1,5,2)
plot(gammaresults)
hold on
xlabel('iters')
ylabel('\gamma')
subplot(1,5,3)
plot(a12results)
hold on
xlabel('iters')
ylabel('\alpha_{12}')
subplot(1,5,4)
plot(a21results)
hold on
xlabel('iters')
ylabel('\alpha_{21}')
subplot(1,5,5)
plot(sigeresults)
hold on
xlabel('iters')
ylabel('\sigma_e')


%%plotting the posterior parameter pdfs (Figure 3d)
figure
subplot(1,5,1)
x=1e-5:1e-9:2.5e-5;
pd1=fitdist(betaresults,'Kernel','Bandwidth',3e-7);
plot(x,pdf(pd1,x),'Linewidth',1.2)
xlabel('\beta'); ylabel('pdf');
subplot(1,5,2)
x=0.55:0.001:1;
pd2=fitdist(gammaresults,'Kernel','bandwidth',0.02);
plot(x,pdf(pd2,x),'Linewidth',1.2)
xlabel('\gamma'); ylabel('pdf');
subplot(1,5,3)
x=0:0.0001:1.2;
pd3=fitdist(a12results,'Kernel','bandwidth',0.05);
plot(x,pdf(pd3,x),'Linewidth',1.2)
xlabel('\alpha_{12}'); ylabel('pdf');
subplot(1,5,4)
x=2:0.0001:4.1;
pd4=fitdist(a21results,'Kernel','bandwidth',0.05);
plot(x,pdf(pd4,x),'Linewidth',1.2)
xlabel('\alpha_{21}'); ylabel('pdf');
subplot(1,5,5)
x=0:0.0001:3;
pd5=fitdist(sigeresults,'Kernel','Bandwidth',0.06);
plot(x,pdf(pd5,x),'Linewidth',1.2)
xlabel('\sigma'); ylabel('pdf');

%%posteriors
BushySposterior=xsampsave(1,:,2:end);
BushyIposterior=xsampsave(2,:,2:end);
BushyRposterior=-BushySposterior-BushyIposterior+Nnode1;
RichmondSposterior=xsampsave(3,:,2:end);
RichmondIposterior=xsampsave(4,:,2:end);
RichmondRposterior=-RichmondSposterior-RichmondIposterior+Nnode2;

%%plotting posteriors (Figure 3a-c)
%Bushy S 
figure
plot(time,mean(BushySposterior,3),'-d','Color',[0 0.4470 0.7410], 'Linewidth',1.2,'MarkerSize',3)
hold on
patch([time fliplr(time)], [quantile(BushySposterior,0.025,3) fliplr(quantile(BushySposterior,0.975,3))],[0, 0.4470, 0.7410],'FaceAlpha',.3,'edgecolor','none')
xlabel('Years')
ylabel('{\itS}_{1,{\itt}}')
xlim([time(1) time(end)])

%Richmond S
figure
plot(time,mean(RichmondSposterior,3),'-o','Color',[0.8500, 0.3250, 0.0980], 'Linewidth',1.2,'MarkerFaceColor','auto','MarkerSize',3,'MarkerFaceColor',[0.8500, 0.3250, 0.0980])
patch([time fliplr(time)], [quantile(RichmondSposterior,0.025,3) fliplr(quantile(RichmondSposterior,0.975,3))],[0.8500, 0.3250, 0.0980],'FaceAlpha',.3,'edgecolor','none')
xlabel('Years')
ylabel('{\itS}_{2,{\itt}}')
xlim([time(1) time(end)])

%Bushy and Richmond I
figure
plot(time,median(BushyIposterior,3),'-d','Color',[0 0.4470 0.7410], 'Linewidth',1.2,'MarkerSize',3)
hold on
patch([time fliplr(time)], [quantile(BushyIposterior,0.025,3) fliplr(quantile(BushyIposterior,0.975,3))],[0, 0.4470, 0.7410],'FaceAlpha',.3,'edgecolor','none')
plot(time,median(RichmondIposterior,3),'-o','Color',[0.8500, 0.3250, 0.0980], 'Linewidth',1.2,'MarkerFaceColor','auto','MarkerSize',3,'MarkerFaceColor',[0.8500, 0.3250, 0.0980])
patch([time fliplr(time)], [quantile(RichmondIposterior,0.025,3) fliplr(quantile(RichmondIposterior,0.975,3))],[0.8500, 0.3250, 0.0980],'FaceAlpha',.3,'edgecolor','none')
xlabel('Years')
ylabel('{\itI}_{1,{\itt}},, {\itI}_{2,{\itt}}')
xlim([time(1) time(end)])

%Bushy and Richmond R
figure
plot(time,median(BushyRposterior,3),'-d','Color',[0 0.4470 0.7410], 'Linewidth',1.2,'MarkerSize',3)
hold on
patch([time fliplr(time)], [quantile(BushyRposterior,0.025,3) fliplr(quantile(BushyRposterior,0.975,3))],[0, 0.4470, 0.7410],'FaceAlpha',.3,'edgecolor','none')
patch([time fliplr(time)], [quantile(RichmondRposterior,0.025,3) fliplr(quantile(RichmondRposterior,0.975,3))],[0.8500, 0.3250, 0.0980],'FaceAlpha',.3,'edgecolor','none')
plot(time,quantile(RichmondRposterior,0.5,3),'-o','Color',[0.8500, 0.3250, 0.0980], 'Linewidth',1.2,'MarkerFaceColor','auto','MarkerSize',3,'MarkerFaceColor',[0.8500, 0.3250, 0.0980])
%plot observations on top
plot(time, BushyData13_21,'--k','Linewidth',1.2)
plot(time, RichmondData13_21,'--k','Linewidth',1.2)
xlim([time(1) time(end)])
xlabel('Years')
ylabel('{\itR}_{1,{\itt}}, {\itR}_{2,{\itt}}')

%%parameter summaries
median(betaresults)
median(gammaresults)
median(a12results)
median(a21results)
median(sigeresults)

mean(betaresults)
mean(gammaresults)
mean(a12results)
mean(a21results)
mean(sigeresults)

std(betaresults)
std(gammaresults)
std(a12results)
std(a21results)
std(sigeresults)

quantile(betaresults,0.025)
quantile(betaresults,0.975)

quantile(gammaresults,0.025)
quantile(gammaresults,0.975)

quantile(sigeresults,0.025)
quantile(sigeresults,0.975)

quantile(a12results,0.025)
quantile(a12results,0.975)

quantile(a21results,0.025)
quantile(a21results,0.975)

%%forward simulation with these parameters 

%generating two sets of simulated data
tf=20; %final time
inter=0.001; %inter observation time
beta=median(betaresults); gam=median(gammaresults); %beta (infection rate) and gamma (removal)
alpha12=median(a12results); alpha21=median(a21results); %transition between node parameters
thetadata=[beta,gam,alpha12,alpha21]; %parameters

%forward simulation for 50 runs
sim=[];
Rtsim1=[]; Rtsim2=[];
Stsim1=[]; Stsim2=[];
Itsim1=[]; Itsim2=[];
for i=1:50
[sim, It1_node1, It1_node2, It2_node2, It2_node1]=SIR2node(Nnode1, Nnode2, I0node1, I0node2, Rtnode1(1), Rtnode2(1), 0, tf, inter, beta, gam, beta, gam, alpha12, alpha21);
Rtsim1(i,:)=Nnode1-sim(:,1)-sim(:,2);
Rtsim2(i,:)=Nnode2-sim(:,3)-sim(:,4);
Stsim1(i,:)=sim(:,1);
Stsim2(i,:)=sim(:,3);
Itsim1(i,:)=sim(:,2);
Itsim2(i,:)=sim(:,4);
newItnode1_node1(i,:)=sum(reshape(It1_node1(2:end),[],tf));
newItnode1_node2(i,:)=sum(reshape(It1_node2(2:end),[],tf));
newItnode2_node2(i,:)=sum(reshape(It2_node2(2:end),[],tf));
newItnode2_node1(i,:)=sum(reshape(It2_node1(2:end),[],tf));
end

%%times for new simulated data
time2=2013:inter:2013+tf;
timeyears=2013:1:2013+tf-1;

%%prediction figures - Figure 6
%%bushy prediction
figure
plot(time2, mean(Rtsim1),'Linewidth',1.2)
hold on
patch([time2 fliplr(time2)], [quantile(Rtsim1,0.025) fliplr(quantile(Rtsim1,0.975))],[0, 0.4470, 0.7410],'FaceAlpha',.3,'edgecolor','none')
hold on 
plot(time, BushyData13_21,'--k','Linewidth',1.2)
ylabel('{\itR}_{1,{\itt}}')
xlabel('Years')
xlim([2013 2025])

%%richmond prediction
figure
plot(time2, mean(Rtsim2),'Color',[0.8500, 0.3250, 0.0980],'Linewidth',1.2)
hold on
patch([time2 fliplr(time2)], [quantile(Rtsim2,0.025) fliplr(quantile(Rtsim2,0.975))],[0.8500, 0.3250, 0.0980],'FaceAlpha',.3,'edgecolor','none')
hold on 
plot(time, RichmondData13_21,'--k','Linewidth',1.2)
%xlim([2013 2013+tf])
ylabel('{\itR}_{2,{\itt}}')
xlabel('Years')
xlim([2013 2025])

%%split into the two components for each park, plotting the new infections (incidence - Figure 5)
%Bushy 
figure
hold on
patch([timeyears fliplr(timeyears)], [quantile(newItnode1_node1,0.025) fliplr(quantile(newItnode1_node1,0.975))],[0, 0.4470, 0.7410],'FaceAlpha',.3,'edgecolor','none')
patch([timeyears fliplr(timeyears)], [quantile(newItnode1_node2,0.025) fliplr(quantile(newItnode1_node2,0.975))],[0, 0.4470, 0.7410],'FaceAlpha',.3,'edgecolor','none')
plot(timeyears, mean(newItnode1_node1),'Linewidth',1.2,'Color',[0, 0.4470, 0.7410])
plot(timeyears, mean(newItnode1_node2),'Linewidth',1.2,'Linestyle','--','Color',[0, 0.4470, 0.7410])
xlabel('Years')
ylabel('New infestations')
xlim([2013 2021])
box on 

%richmond park
figure
hold on
patch([timeyears fliplr(timeyears)], [quantile(newItnode2_node1,0.025) fliplr(quantile(newItnode2_node1,0.975))],[0.8500, 0.3250, 0.0980],'FaceAlpha',.3,'edgecolor','none')
patch([timeyears fliplr(timeyears)], [quantile(newItnode2_node2,0.025) fliplr(quantile(newItnode2_node2,0.975))],[0.8500, 0.3250, 0.0980],'FaceAlpha',.3,'edgecolor','none')
plot(timeyears, mean(newItnode2_node2),'Linewidth',1.2, 'Color',[0.8500, 0.3250, 0.0980])
plot(timeyears, mean(newItnode2_node1),'Linewidth',1.2,'Linestyle','--','Color',[0.8500, 0.3250, 0.0980])
xlabel('Years')
ylabel('New infestations')
xlim([2013 2021])
box on 

%%plotting distributions of beta*alpha12*I_1*S_2 (Figure 4)
medianBushyI=median(BushyIposterior,3);
medianBushyS=median(BushySposterior,3);
medianRichmondI=median(RichmondIposterior,3);
medianRichmondS=median(RichmondSposterior,3);

%%NODE 1 
%Bushy Park
%normal intra park contribution
figure
for i=1:length(years)
y2=betaresults.*medianBushyI(i).*medianBushyS(i);
x=0:0.01:35;
pd2=fitdist(y2,'Kernel');
plot(x,pdf(pd2,x),'Linewidth',1.2)
hold on 
end 
ylabel('pdf')
xlim([0 35])
xlabel('\beta{\itI}_1{\itS}_1')

%%Bushy Park
%%inter-park contribution
figure
for i=1:length(years)
y1=betaresults.*a21results.*medianRichmondI(i).*medianBushyS(i);
x=0:1:450;
pd1=fitdist(y1,'Kernel');
plot(x,pdf(pd1,x),'Linewidth',1.2)
hold on
end 
ylabel('pdf')
xlim([0 450])
xlabel('\beta\alpha_{21}{\itI}_2{\itS}_1')

%%Node 2
%Richmond Park
%normal intra-park contribution
figure
for i=1:length(years)
y2=betaresults.*medianRichmondI(i).*medianRichmondS(i);
x=0:1:1100;
pd2=fitdist(y2,'Kernel');
hold on
plot(x,pdf(pd2,x),'Linewidth',1.2)
end 
ylabel('pdf')
xlabel('\beta{\itI}_2{\itS}_2')
box on 

%%Richmond Park
%%inter-park contribution
figure
for i=1:length(years)
y1=betaresults.*a12results.*medianBushyI(i).*medianRichmondS(i);
x=0:1:220;
pd1=fitdist(y1,'Kernel');
hold on
plot(x,pdf(pd1,x),'Linewidth',1.2)
end 
ylabel('pdf')
xlabel('\beta\alpha_{12}{\itI}_1{\itS}_2')
box on 
xlim([0 220])
