function [z,V,P] = tstep2node(T,dt,initz,initV,theta)
%function which steps forward the governing ODE using the LNA

%LNA splits up Xt into a deterministic and a residual process
%in this code, this is Xt=z_t+R_t
%the deterministic part z_t satisfies dz_t/dt=alpha(z_t,theta)
%it can be shown (using properties of R_t), that Xt~N(z_t,V_t)
%where dz_t/dt=alpha(z_t,theta) and dV_t/dt=VtFmat'+beta(z_t,theta)+FmatV)
%and Fmat is the jacobian of alpha, stored here in Fmat
%Pt is fundamental matrix required for dPt=FmatPt 

%z: deterministic process
%V: stochastic process (variance)
%P: fundamental matrix 

n=T/dt;
d=length(initz);
z=zeros(d,n+1); %store z solution (solution to ODE, S1,I1,S2,I2)
V=zeros(d,d,n+1); %store V solution here (variance of stoch)
P=zeros(d,d,n+1); %store P solution here (fundamental matrix solution of dPt=Fmat*Pt

%initialise
z(:,1)=initz;
V(:,:,1)=initV;
P(:,:,1)=eye(d);

%step forward
for i=2:n+1
    Fm=FmatSIR2node(z(:,i-1),theta); %Fmat from LNA
    z(:,i)=z(:,i-1)+alphaSIR2node(z(:,i-1),theta)'*dt; %z solution
    
    %conditions on S and I (if S small and I small)
    if z(1,i)<0.001 
        z(1,i)=0.001; %replace with 0.01
    end  
    if z(2,i)<0.001 
        z(2,i)=0.001; %replace with 0.01
    end 
    if z(3,i)<0.001 
        z(3,i)=0.001; %replace with 0.01
    end  
    if z(4,i)<0.001 
        z(4,i)=0.001; %replace with 0.01
    end 
    
    P(:,:,i)=P(:,:,i-1)+dt*Fm*P(:,:,i-1); %mean solution
    V(:,:,i)=V(:,:,i-1)+(V(:,:,i-1)*Fm'+betaSIR2node(z(:,i-1),theta)+Fm*V(:,:,i-1))*dt; %variance solution
end 

end 
